/*******************************************************
This project was created by the http://www.uboard.ir

Author  		: Reza Asadi
IDE 			: Atmel Studio
Project 		: PWM Using PWM mode 7 of timer/counter1
Version 		: 1
Date    		: 16/11/2020
Chip type               : ATmega64A
Core Clock frequency	: 8.000000 MHz
Comments		: This project is a exercise for making
				  variable frequency PWM with timer.
*******************************************************/
#define F_CPU 8000000UL

#define next_pwm_change_key				(PINE & (1<<7))
#define duty_cycle_decrease_key			(PINE & (1<<4))
#define duty_cycle_increase_key			(PINE & (1<<5))
#define frequency_decrease_key			(PINE & (1<<6))
#define frequency_increase_key			(PINB & (1<<0))

#define oc1a_pwm_channel	0
#define oc1b_pwm_channel	1
#define oc1c_pwm_channel	2
#define pa0_pwm_channel		3


#define timer1_mode_7_top_value		1023
#define timer0_mode_0_top_value		255
#define timer1_clock_frequency		1000000
#define timer0_clock_frequency		1000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include "Libs/clcd.h"

unsigned char freq1=5, freq0=5; // frequencies in kHz
unsigned int tcnt1_start_value=  (unsigned int)  (timer1_mode_7_top_value - (timer1_clock_frequency/(freq1*1000)));
unsigned char tcnt0_start_value= (unsigned char) (timer0_mode_0_top_value - (timer0_clock_frequency/(freq0*1000)));


ISR(TIMER1_OVF_vect)
{
	TCNT1=tcnt1_start_value;
}

ISR(TIMER0_OVF_vect)
{
	TCNT0=tcnt0_start_value;
	PORTA |= (1<<0);  // set PA.0 at overflow
}

ISR(TIMER0_COMP_vect)
{
	PORTA &=~ (1<<0);  // clear PA.0 at Compare Match
}

int main(void)
{
	char oc1a_duty_cycle=10, oc1b_duty_cycle=10, oc1c_duty_cycle=10;
	char pa0_duty_cycle=10;
	unsigned int ocr1a_value= (unsigned int) (tcnt1_start_value + ((oc1a_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
	unsigned int ocr1b_value= (unsigned int) (tcnt1_start_value + ((oc1b_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
	unsigned int ocr1c_value= (unsigned int) (tcnt1_start_value + ((oc1c_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
	unsigned char ocr0_value= (unsigned char)(tcnt0_start_value + ((pa0_duty_cycle  * 0.01) * (timer0_mode_0_top_value - tcnt0_start_value)));
	
	unsigned char pwm_channel=oc1a_pwm_channel;
	char lcd_str[21];
	
	DDRB=(1<<DDB7) | (1<<DDB6) | (1<<DDB5);
	DDRA |= (1<<DDA0);
	PORTE |= (1<<7)|(1<<6)|(1<<5)|(1<<4);
	PORTB |= (1<<0);
	
	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 1000.000 kHz
	// Mode: Fast PWM top=0x03FF=1023
	// OC1A/B/C output: Non-Inverted PWM
	// Output Pulse(s):
	// OC1A Period: 1 ms Width: 0.1 ms
	// OC1B Period: 1 ms Width: 0.1 ms
	// OC1C Period: 1 ms Width: 0.1 ms
	TCCR1A=(1<<COM1A1) | (0<<COM1A0) | (1<<COM1B1) | (0<<COM1B0) | (1<<COM1C1) | (0<<COM1C0) | (1<<WGM11) | (1<<WGM10);
	TCCR1B=(0<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (1<<WGM12) | (0<<CS12) | (1<<CS11) | (0<<CS10);
	TCNT1=tcnt1_start_value;
	OCR1AH= ocr1a_value >> 8;
	OCR1AL= ocr1a_value & 0xFF;
	OCR1BH= ocr1b_value >> 8;
	OCR1BL= ocr1b_value & 0xFF;
	OCR1CH= ocr1c_value >> 8;
	OCR1CL= ocr1c_value & 0xFF;
	
	// Timer1 Overflow Interrupt: On
	TIMSK |= (1<<TOIE1);
	
	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: 1000.000 kHz
	// Mode: Normal top=0xFF
	// OC0 output: Disconnected
	ASSR=0<<AS0;
	TCCR0=(0<<WGM00) | (0<<COM01) | (0<<COM00) | (0<<WGM01) | (0<<CS02) | (1<<CS01) | (0<<CS00);
	TCNT0=tcnt0_start_value;
	OCR0=ocr0_value;

	// Timer0 Overflow Interrupt: On
	// Timer0 Compare Match Interrupt: On
	TIMSK |= (1<<OCIE0) | (1<<TOIE0);
	
	sei();
	
	lcd_init();
	lcd_clear();
	lcd_puts("UBOARD.ir");
	_delay_ms(1500);
	
	while (1)
	{
		if (!next_pwm_change_key)
		{
			_delay_ms(50);
			while (!next_pwm_change_key);
			if      (pwm_channel==oc1a_pwm_channel) pwm_channel=oc1b_pwm_channel;
			else if (pwm_channel==oc1b_pwm_channel) pwm_channel=oc1c_pwm_channel;
			else if (pwm_channel==oc1c_pwm_channel) pwm_channel=pa0_pwm_channel;
			else if (pwm_channel==pa0_pwm_channel)  pwm_channel=oc1a_pwm_channel;
		}
		else if (!duty_cycle_decrease_key)
		{
			_delay_ms(50);
			while (!duty_cycle_decrease_key);
			if(pwm_channel==oc1a_pwm_channel)
			{
				oc1a_duty_cycle-=10;
				if(oc1a_duty_cycle<10) oc1a_duty_cycle=10;
				ocr1a_value= (unsigned int) (tcnt1_start_value + ((oc1a_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				OCR1AH= ocr1a_value >> 8;
				OCR1AL= ocr1a_value & 0xFF;
			}
			else if (pwm_channel==oc1b_pwm_channel)
			{
				oc1b_duty_cycle-=10;
				if(oc1b_duty_cycle<10) oc1b_duty_cycle=10;
				ocr1b_value= (unsigned int) (tcnt1_start_value + ((oc1b_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				OCR1BH= ocr1b_value >> 8;
				OCR1BL= ocr1b_value & 0xFF;
			}
			else if (pwm_channel==oc1c_pwm_channel)
			{
				oc1c_duty_cycle-=10;
				if(oc1c_duty_cycle<10) oc1c_duty_cycle=10;
				ocr1c_value= (unsigned int) (tcnt1_start_value + ((oc1c_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				OCR1CH= ocr1c_value >> 8;
				OCR1CL= ocr1c_value & 0xFF;
			}
			else if (pwm_channel==pa0_pwm_channel)
			{
				pa0_duty_cycle-=10;
				if(pa0_duty_cycle<10) pa0_duty_cycle=10;
				ocr0_value= (unsigned char)(tcnt0_start_value + ((pa0_duty_cycle  * 0.01) * (timer0_mode_0_top_value - tcnt0_start_value)));
				OCR0=ocr0_value;
			}
		}
		else if (!duty_cycle_increase_key)
		{
			_delay_ms(50);
			while (!duty_cycle_increase_key);
			if(pwm_channel==oc1a_pwm_channel)
			{
				oc1a_duty_cycle+=10;
				if(oc1a_duty_cycle>90) oc1a_duty_cycle=90;
				ocr1a_value= (unsigned int) (tcnt1_start_value + ((oc1a_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				OCR1AH= ocr1a_value >> 8;
				OCR1AL= ocr1a_value & 0xFF;
			}
			else if (pwm_channel==oc1b_pwm_channel)
			{
				oc1b_duty_cycle+=10;
				if(oc1b_duty_cycle>90) oc1b_duty_cycle=90;
				ocr1b_value= (unsigned int) (tcnt1_start_value + ((oc1b_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				OCR1BH= ocr1b_value >> 8;
				OCR1BL= ocr1b_value & 0xFF;
			}
			else if (pwm_channel==oc1c_pwm_channel)
			{
				oc1c_duty_cycle+=10;
				if(oc1c_duty_cycle>90) oc1c_duty_cycle=90;
				ocr1c_value= (unsigned int) (tcnt1_start_value + ((oc1c_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				OCR1CH= ocr1c_value >> 8;
				OCR1CL= ocr1c_value & 0xFF;
			}
			else if (pwm_channel==pa0_pwm_channel)
			{
				pa0_duty_cycle+=10;
				if(pa0_duty_cycle>90) pa0_duty_cycle=90;
				ocr0_value= (unsigned char)(tcnt0_start_value + ((pa0_duty_cycle  * 0.01) * (timer0_mode_0_top_value - tcnt0_start_value)));
				OCR0=ocr0_value;
			}
		}
		else if (!frequency_decrease_key)
		{
			_delay_ms(50);
			while (!frequency_decrease_key);
			if((pwm_channel==oc1a_pwm_channel)||(pwm_channel==oc1b_pwm_channel)||(pwm_channel==oc1c_pwm_channel))
			{
				freq1--;
				if(freq1<1) freq1=1;
				tcnt1_start_value=  (unsigned int)  (timer1_mode_7_top_value - (timer1_clock_frequency/(freq1*1000)));
				ocr1a_value= (unsigned int) (tcnt1_start_value + ((oc1a_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				ocr1b_value= (unsigned int) (tcnt1_start_value + ((oc1b_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				ocr1c_value= (unsigned int) (tcnt1_start_value + ((oc1c_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				OCR1AH= ocr1a_value >> 8;
				OCR1AL= ocr1a_value & 0xFF;
				OCR1BH= ocr1b_value >> 8;
				OCR1BL= ocr1b_value & 0xFF;
				OCR1CH= ocr1c_value >> 8;
				OCR1CL= ocr1c_value & 0xFF;
			}
			else if(pwm_channel==pa0_pwm_channel)
			{
				freq0--;
				if(freq0<4) freq0=4;
				tcnt0_start_value= (unsigned char) (timer0_mode_0_top_value - (timer0_clock_frequency/(freq0*1000)));
				ocr0_value= (unsigned char)(tcnt0_start_value + ((pa0_duty_cycle  * 0.01) * (timer0_mode_0_top_value - tcnt0_start_value)));
				OCR0=ocr0_value;
			}
		}
		else if (!frequency_increase_key)
		{
			_delay_ms(50);
			while (!frequency_increase_key);
			if((pwm_channel==oc1a_pwm_channel)||(pwm_channel==oc1b_pwm_channel)||(pwm_channel==oc1c_pwm_channel))
			{
				freq1++;
				if(freq1>10) freq1=10;
				tcnt1_start_value=  (unsigned int)  (timer1_mode_7_top_value - (timer1_clock_frequency/(freq1*1000)));
				ocr1a_value= (unsigned int) (tcnt1_start_value + ((oc1a_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				ocr1b_value= (unsigned int) (tcnt1_start_value + ((oc1b_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				ocr1c_value= (unsigned int) (tcnt1_start_value + ((oc1c_duty_cycle * 0.01) * (timer1_mode_7_top_value - tcnt1_start_value)));
				OCR1AH= ocr1a_value >> 8;
				OCR1AL= ocr1a_value & 0xFF;
				OCR1BH= ocr1b_value >> 8;
				OCR1BL= ocr1b_value & 0xFF;
				OCR1CH= ocr1c_value >> 8;
				OCR1CL= ocr1c_value & 0xFF;
			}
			else if(pwm_channel==pa0_pwm_channel)
			{
				freq0++;
				if(freq0>10) freq0=10;
				tcnt0_start_value= (unsigned char) (timer0_mode_0_top_value - (timer0_clock_frequency/(freq0*1000)));
				ocr0_value= (unsigned char)(tcnt0_start_value + ((pa0_duty_cycle  * 0.01) * (timer0_mode_0_top_value - tcnt0_start_value)));
				OCR0=ocr0_value;
			}
		}
		
		lcd_clear();
		lcd_gotoxy(0,0);
		if(pwm_channel==oc1a_pwm_channel) lcd_puts("Change OC1A Values!");
		if(pwm_channel==oc1b_pwm_channel) lcd_puts("Change OC1B Values!");
		if(pwm_channel==oc1c_pwm_channel) lcd_puts("Change OC1C Values!");
		if(pwm_channel==pa0_pwm_channel)  lcd_puts("Change PA0 Values!");
		lcd_gotoxy(0,1);
		lcd_puts("OC..1A.1B.1C  PA0");		
		lcd_gotoxy(0,2);
		sprintf(lcd_str,"f=  %.2d %.2d %.2d  %.2d kHz",freq1,freq1,freq1,freq0);
		lcd_puts(lcd_str);
		lcd_gotoxy(0,3);
		sprintf(lcd_str,"DC= %.2d %.2d %.2d  %.2d %%",oc1a_duty_cycle,oc1b_duty_cycle,oc1c_duty_cycle,pa0_duty_cycle);
		lcd_puts(lcd_str);
		_delay_ms(50);
	}
}